;*************************************************************** 
;* Feladat: rjon egy programot az AVR Experiment krtyra, amely 
;  gombnyomsra lpteti a fnyt a LEDsoron. Bekapcsols utn a 
;  program gyjtsa ki a LED4 didt, a tbbi ne vilgtson. A
;  vilgt LED-pozci a BT0 s BT2 nyomgombok segtsgvel 
;  lptetheto, BT0-val az egyik irnyba, BT2-vel a msikba. 
;  Minden gombnyoms 1 pozcinyit lptet az adott irnyba. Ha a
;  lptets sorn elrtk a szls helyzetet, akkor ne lehessen 
;  kilptetni a vilgt LED-et. A nyomgomb pergsmentestst 
;  oldja meg. 
;
;* Rvid lers: LED-lptets
; 
;* Szerzk: Okos Dniel
;* Mrcsoport: <merocsoport jele>
;
;***************************************************************
;* "AVR ExperimentBoard" port assignment information:
;***************************************************************
;*
;* LED0(P):PortC.0          LED4(P):PortC.4
;* LED1(P):PortC.1          LED5(P):PortC.5
;* LED2(S):PortC.2          LED6(S):PortC.6
;* LED3(Z):PortC.3          LED7(Z):PortC.7        INT:PortE.4
;*
;* SW0:PortG.0     SW1:PortG.1     SW2:PortG.4     SW3:PortG.3
;* 
;* BT0:PortE.5     BT1:PortE.6     BT2:PortE.7     BT3:PortB.7
;*
;***************************************************************
;*
;* AIN:PortF.0     NTK:PortF.1    OPTO:PortF.2     POT:PortF.3
;*
;***************************************************************
;*
;* LCD1(VSS) = GND         LCD9(DB2): -
;* LCD2(VDD) = VCC         LCD10(DB3): -
;* LCD3(VO ) = GND         LCD11(DB4): PortA.4
;* LCD4(RS ) = PortA.0     LCD12(DB5): PortA.5
;* LCD5(R/W) = GND         LCD13(DB6): PortA.6
;* LCD6(E  ) = PortA.1     LCD14(DB7): PortA.7
;* LCD7(DB0) = -           LCD15(BLA): VCC
;* LCD8(DB1) = -           LCD16(BLK): PortB.5 (1=Backlight ON)
;*
;***************************************************************

.include "m128def.inc" ; Definition file for ATmega128 
;* Program Constants 
.equ const =$00 ; Generic Constant Structure example  
;* Program Variables Definitions 
.def temp 	=r16 ; Temporary Register example 
.def pressed0	=r17 ; BT0 lenyomst jelzo "flag"
.def pressed2	=r18 ; BT2 lenyomst jelzo "flag"
.def pattern0 	=r19 ; BT0-hoz tartoz bitminta
.def pattern2 	=r20 ; BT2-hz tartoz bitminta

;*************************************************************** 
;* Reset & Interrupt Vectors  
.cseg 
.org $0000 ; Define start of Code segment 
	jmp RESET ; Reset Handler, jmp is 2 word instruction 
	jmp DUMMY_IT	; Ext. INT0 Handler
	jmp DUMMY_IT	; Ext. INT1 Handler
	jmp DUMMY_IT	; Ext. INT2 Handler
	jmp DUMMY_IT	; Ext. INT3 Handler
	jmp DUMMY_IT	; Ext. INT4 Handler (INT gomb)
	jmp DUMMY_IT	; Ext. INT5 Handler
	jmp DUMMY_IT	; Ext. INT6 Handler
	jmp DUMMY_IT	; Ext. INT7 Handler
	jmp DUMMY_IT	; Timer2 Compare Match Handler 
	jmp DUMMY_IT	; Timer2 Overflow Handler 
	jmp DUMMY_IT	; Timer1 Capture Event Handler 
	jmp DUMMY_IT	; Timer1 Compare Match A Handler 
	jmp DUMMY_IT	; Timer1 Compare Match B Handler 
	jmp DUMMY_IT	; Timer1 Overflow Handler 
	jmp TIMER_IT	; Timer0 Compare Match Handler 
	jmp DUMMY_IT	; Timer0 Overflow Handler 
	jmp DUMMY_IT	; SPI Transfer Complete Handler 
	jmp DUMMY_IT	; USART0 RX Complete Handler 
	jmp DUMMY_IT	; USART0 Data Register Empty Hanlder 
	jmp DUMMY_IT	; USART0 TX Complete Handler 
	jmp DUMMY_IT	; ADC Conversion Complete Handler 
	jmp DUMMY_IT	; EEPROM Ready Hanlder 
	jmp DUMMY_IT	; Analog Comparator Handler 
	jmp DUMMY_IT	; Timer1 Compare Match C Handler 
	jmp DUMMY_IT	; Timer3 Capture Event Handler 
	jmp DUMMY_IT	; Timer3 Compare Match A Handler 
	jmp DUMMY_IT	; Timer3 Compare Match B Handler 
	jmp DUMMY_IT	; Timer3 Compare Match C Handler 
	jmp DUMMY_IT	; Timer3 Overflow Handler 
	jmp DUMMY_IT	; USART1 RX Complete Handler 
	jmp DUMMY_IT	; USART1 Data Register Empty Hanlder 
	jmp DUMMY_IT	; USART1 TX Complete Handler 
	jmp DUMMY_IT	; Two-wire Serial Interface Handler 
	jmp DUMMY_IT	; Store Program Memory Ready Handler 

.org $0046

;****************************************************************
;* DUMMY_IT interrupt handler -- CPU hangup with LED pattern
;* (This way unhandled interrupts will be noticed)

;< tbbi IT kezel a fjl vgre! >

DUMMY_IT:	
	ldi r16,   0xFF ; LED pattern:  *-
	out DDRC,  r16  ;               -*
	ldi r16,   0xA5	;               *-
	out PORTC, r16  ;               -*
DUMMY_LOOP:
	rjmp DUMMY_LOOP ; endless loop

;< tbbi IT kezel a fjl vgre! >

;*************************************************************** 
;* MAIN program, Initialisation part
.org $004B;
RESET: 
;* Stack Pointer init, 
;  Set stack pointer to top of RAM 
	ldi temp, LOW(RAMEND) ; RAMEND = "max address in RAM"
	out SPL, temp 	      ; RAMEND value in "m128def.inc" 
	ldi temp, HIGH(RAMEND) 
	out SPH, temp 

M_INIT:
;< ki- s bemenetek inicializlsa stb > 
	; LED init
	ldi temp, 0xFF		; irny - kimenet
	out DDRC, temp
	ldi temp, 0x10		; LED4 vilgt
	out PORTC, temp

	; BTN init
	ldi temp, 0x00		; irny - bemenet
	out DDRE, temp


	; TIMER - csak pergsmentestshez
	ldi temp, 0x0f		; 1024-es elooszt -> 10800Hz
	out TCCR0, temp	
	ldi temp, 107		; 108 ciklus -> 10800/108 = 100Hz
	out OCR0, temp
	ldi temp, 0x02		; TIMER IT engedlyezs
	out TIMSK, temp

	ldi pressed0, 0		; BT0 nincs lenyomva
	ldi pattern0, 0xff	; bitminta kezdetben csupa 1
	ldi pressed2, 0		; BT2 nincs lenyomva
	ldi pattern2, 0xff	; bitminta kezdetben csupa 1

	sei			; globlis IT engedlyezs

;*************************************************************** 
;* MAIN program, Endless loop part
 
M_LOOP: 

;< fciklus >

	tst pressed0		; BT0 test for zero
	breq PRESSED_2		; ha nincs gombnyoms, ugrunk
	clr pressed0		; gombnyoms esetn flag trlse
	call LEFT		; balra lptets

PRESSED_2:
	tst pressed2		; BT2 test for zero
	breq M_LOOP		; ha nincs gombnyoms, ugrunk
	clr pressed2		; gombnyoms esetn flag trlse
	call RIGHT		; jobbra lptets

	jmp M_LOOP ; Endless Loop  


;*************************************************************** 
;* Subroutines, Interrupt routines

TIMER_IT:			; TIMER IT rutin
	push temp		; ments
	in temp, SREG
	push temp

	; pergsmentests
	in temp, PINE		; gombok beolvassa
	bst temp, 5		; BT0 llapot eltrolsa
	lsl pattern0		; bitminta shiftelse balra
	bld pattern0, 0		; az LSB helyre beszrjuk 
				; a gomb llapott jelzo bitet

	andi pattern0, 0x0f	; felso 4 bitet eldobjuk
	cpi pattern0, 0b00001100; 1100 bitminta?
	brne NOT_PRESSED0	; ha nem, nincs gombnyoms,
				; vagy csak pergst szlel
	inc pressed0		; ha igen, tnyleg lefut l

NOT_PRESSED0:

	bst temp, 7		; BT2 llapot eltrolsa
	lsl pattern2		; bitminta shiftelse balra
	bld pattern2, 0		; az LSB helyre beszrjuk 
				; a gomb llapott jelzo bitet

	andi pattern2, 0x0f	; felso 4 bitet eldobjuk
	cpi pattern2, 0b00001100; 1100 bitminta?
	brne NOT_PRESSED2	; ha nem, nincs gombnyoms,
				; vagy csak pergst szlel
	inc pressed2		; ha igen, tnyleg lefut l	

NOT_PRESSED2:
	pop temp		; visszallts
	out SREG, temp
	pop temp
	reti


LEFT:				; balra lptets
	in temp, PINC		; LED llapot beolvass
	sbrs temp, 7		; skip, ha LED7-nl tartunk
				;  hogy ne lehessen kilptetni
	lsl temp		; klnben balra shift
	out PORTC, temp		; kijelzs
	jmp M_LOOP

RIGHT:
	in temp, PINC		; LED llapot beolvass
	sbrs temp, 0		; skip, ha LED0-nl tartunk
	lsr temp		; klnben jobbra shift
	out PORTC, temp		; kijelzs
	jmp M_LOOP


